import { Injectable } from "@angular/core";
import { MermaidUtil, PbBassin, PbBassinParams, PbCloison, PreBarrage } from "jalhyd";

/**
 * service relatif au schéma de prébarrage
 */
@Injectable()
export class PrebarrageService {
    /** predam data */
    private _model: PreBarrage;

    public readonly upstreamId = "amont";

    public readonly downstreamId = "aval";

    public get model(): PreBarrage {
        return this._model;
    }

    public set model(pb: PreBarrage) {
        this._model = pb;
    }

    /**
     * return upstream (and downstream) basin object
     */
    public get upstreamBassin(): PbBassin {
        return (this._model as unknown) as PbBassin;
    }

    public deleteSelected(selectedItem: PbCloison | PbBassin, emptyFields: boolean) {
        this._model.deleteChild(selectedItem.findPositionInParent());
        // never let an unconnected basin ! (not done in model to prevent unwanted
        // automatic child addition when clearing children)
        if (selectedItem instanceof PbCloison) {
            // if no downstream connections remain, connect to river downstream
            if (selectedItem.bassinAmont?.cloisonsAval.length === 0) {
                this._model.addChild(new PbCloison(selectedItem.bassinAmont, undefined, undefined, emptyFields));
            }
            // if no upstream connections remain, connect to river upstream
            if (selectedItem.bassinAval?.cloisonsAmont.length === 0) {
                this._model.addChild(new PbCloison(undefined, selectedItem.bassinAval, undefined, emptyFields));
            }
        }
    }

    public copyWall(wall: PbCloison, emptyFields: boolean): PbCloison {
        const wallCopy = new PbCloison(wall.bassinAmont, wall.bassinAval, undefined, emptyFields);
        wallCopy.loadObjectRepresentation(wall.objectRepresentation());
        this._model.addChild(wallCopy);
        return wallCopy;
    }

    public addBasin(emptyFields: boolean): PbBassin {
        const newBasin = new PbBassin(new PbBassinParams(20, 99, emptyFields));
        this._model.addChild(newBasin);
        return newBasin;
    }

    public get hasBasins(): boolean {
        return this._model.bassins.length > 0;
    }

    public get bassins(): PbBassin[] {
        return this._model.bassins;
    }

    public addWall(upstreamIndex: number, downstreamIndex: number, emptyFields: boolean): PbCloison {
        const wall = new PbCloison(
            upstreamIndex === 0 ? undefined : this._model.bassins[upstreamIndex - 1],
            downstreamIndex === 0 ? undefined : this._model.bassins[downstreamIndex - 1],
            undefined,
            emptyFields
        );
        this._model.addChild(wall);
        return wall;
    }

    /**
     * Returns true if given basin is either connected to nothing, or only to
     * river upstream or downstream
     */
    public isStandaloneBasin(basin: PbBassin) {
        return (
            (
                basin.cloisonsAmont.length === 0
                || basin.cloisonsAmont.map(c => c.bassinAmont).every(e => e === undefined)
            ) && (
                basin.cloisonsAval.length === 0
                || basin.cloisonsAval.map(c => c.bassinAval).every(e => e === undefined)
            )
        );
    }

    /**
     * @param uid nub uid
     */
    public findBasinPosition(uid: string): number {
        return this._model.findBasinPosition(uid);
    }

    /**
     * @param uid nub uid
     */
    public moveBasinUp(uid: string) {
        this._model.moveBasin(uid, this._model.findBasinPosition(uid) - 1);
    }

    /**
     * @param uid nub uid
     */
    public moveBasinDown(uid: string) {
        this._model.moveBasin(uid, this._model.findBasinPosition(uid) + 1);
    }

    /**
     * @param uid nub uid
     */
    public isLastBasin(uid: string): boolean {
        return this._model.findBasinPosition(uid) === this._model.bassins.length - 1
    }

    public isValid(): boolean {
        return this._model.hasUpDownConnection() && !this._model.hasBasinNotConnected();
    }

    /**
     * @param uid nub uid
     */
    public findChild(uid: string): PbBassin | PbCloison {
        return this._model.findChild(uid);
    }

    /**
     * turn 'aval', 'amont' and Mermaid ids to nub real uid
     */
    public toNubUid(itemId: string): string {
        if (itemId !== undefined && itemId !== null) {
            if (this.matchMermaidIds([this.upstreamId, this.downstreamId], itemId)) {
                return this.model.uid;
            }
            if (itemId.startsWith("flowchart-")) { // Mermaid style id (ie. flowchart-id-xx)
                return itemId.split('-')[1];
            }
        }
        return itemId;
    }

    /**
     * check if a list of (Mermaid transformed) ids matches a given id
     * @param ids ids to transform the Mermaid way
     * @param itemId id to find
     */
    private matchMermaidIds(ids: string[], itemId: string): boolean {
        return ids.find(id => MermaidUtil.isMermaidEqualIds(id, itemId)) !== undefined;
    }

    /**
     * @param itemId Mermaid id to find
     * @returns 
     */
    public findFromItemId(itemId: string) {
        if (this.matchMermaidIds([this.upstreamId, this.downstreamId], itemId)) {
            return this.upstreamBassin;
        } else {
            return this.model.findChild(itemId);
        }
    }
}
